<?php
/**
* Edycja wersji jzykowych
*
* @author  lech@sote.pl
* @version $Id: lang_editor.inc.php,v 1.4 2004/05/14 10:47:06 maroslaw Exp $
* @package lang_editor
*/

/**
* @todo usunac polskie opisy z kodu
*/

/**
* @ignore
* @package lang_editor
*/
class LangTmp {};

/**
* Edycja wersji jzykowych
*
* Klasa implementuje metody aktualizacji wersji jzykowych we wszystkichplikach "lang"
* w katalogu htdocs i gbiej
* @package lang_editor
*/
class LangEditor{

    /**
    * Tablica plikw z definicjami jzykw
    *
    * Tablica zawiera list elementw tablicowych: element posiada dwa pola:
    * <ul>
    *   <li>location - tablica, ktrej klucze to skrtowe nazwy jzykw,
    *       a wartoci - lokalizacje plikw,</li>
    *   <li>group - nazwa moduu (grupy), do ktrego nale zmienne definiowane przez plik</li>
    * </ul>
    * Tablica inicjowana jest przez konstruktor.
    */
    var $fileArray;

    /**
    * Tablica dostpnych jzykw
    *
    * Kady element tablicy to skrtowa nazwa jzyka, np. "en", "pl", "de" itp.
    * Inicjacja przez konstruktor (domylne wartoci to: "pl", "en", "de").
    */
    var $languages;

    /**
    * Tablica moduw (grup) systemu, w ktrych mog by pliki z definicjami jzykw
    *
    * Moduy s tu rozumiane jako jednostki systemu porozmieszczane w katalogach
    * "go" lub "plugins" w podkatalogach, ktrych nazwa zaczyna si od znaku "_".
    * Nazw moduu jest nazwa takiego podkatalogu, ale bez znaku "_".
    * Jeden z moduw nazywa si "main": gromadzi on zmienne jzykowe znajdujce si
    * w globalnych plikach jzykowych (czyli htdocs/lang/).
    * Inicjacja przez konstruktor.
    */
    var $groups = array();
    
    /**
    * cieka do katalogu tymczasowego
    *
    * cieka do katalogu tymczasowego, do ktrego kopiowane s tymczasowo pliki
    * z definicjami jzykowymi.
    */
    var $temp_dir;
    
    /**
    * Tablica ze zmiennymi z aktualnie wczytanego pliku
    *
    * Tablica zawiera nazwy i wartoci zmiennych z aktualnie analizowanego pliku
    * z definicjami jzykowymi. Klucz pojedynczego elementu to nazwa zdefiniowanej
    * zmiennej. Struktura pojedynczego elementu tablicy to tablica zawierajca
    * nastpujce pola:
    * <ul>
    * <li>value - tablica, ktrej klucze to skrtowe nazwy jzykw,
    *     a wartoci - wartoci zmiennych</li>
    * <li>group - modu (grupa), do ktrej naley zmienna: w wikszoci przypadkw pole to
    * zdeterminowane jest przez plik, z ktrego pochodzi zmienna, jednak w przypadku
    * zmiennych z gwnego pliku jzykowego s one bd przypisywane do grupy "main",
    * bd do innej grupy, jeli nazwa tej grupy pokrywa si z pocztkiem nazwy zmiennej</li>
    * <li>location  - tablica, ktrej klucze to skrtowe nazwy jzykw,
    *     a wartoci - lokalizacje plikw, w ktrych zdefiniowane s zmienne</li>
    * </ul>
    * Inicjacja przez konstruktor.
    */
    var $currentVars = array();
    
    /**
    * Konstruktor klasy
    *
    * Konstruktor inicjuje pola klasy.
    * 
    * @param string $lang_list_file cieka do pliku zawierajcego cieki do
    * wszystkich plikw z POLSKIMI definicjami jzykowymi. Przykad poprawnej
    * pojedynczej lini pliku: ./htdocs/go/_contact/lang/_pl/lang.inc.php (istotny
    * jest pocztek cieki "./htdocs/" jak i jej koniec "/_pl/lang.inc.php".
    *
    * @param string $langs Lista skrtowych nazw jzykw podanych po przecinku.
    * Domylna warto parametru: "pl, en, de" .
    */
    function LangEditor($lang_list_file, $langs = "pl, en, de"){
        global $HTTP_SERVER_VARS;
        $DOCUMENT_ROOT = $HTTP_SERVER_VARS['DOCUMENT_ROOT'];
        $this->temp_dir = $DOCUMENT_ROOT . '/../admin/tmp';
        $this->languages = explode( ',', str_replace(' ', '', $langs));
        if(is_file($lang_list_file)){
            $handle = fopen($lang_list_file,'r');
/*
            while (!feof($handle)){
                $line = trim(fgets($handle));
                if($line != '')
                $buffer[] = $line;
            }
*/
            /*
            while (!feof($handle)){
                $line = trim(fgets($handle));
                if($line != '')
                $buffer[] = $line;
            }
            fclose($handle);
            */
            $lines=fread($handle,filesize($lang_list_file));
            $buffer=split("\n",$lines);
            fclose($handle);
            for($i = 0; $i < count($buffer); $i++){
                $groupname = strstr($buffer[$i], '_');
                $groupname = substr($groupname, 1, strpos($groupname, '/') - 1);
                if(!in_array($groupname, $this->groups) && ($groupname != 'pl')){
                    $this->groups[] = $groupname;
                }
                if($groupname == 'pl')
                    $groupname = 'main';
                $elem = array();
                for($j = 0; $j < count($this->languages); $j++){
                    $elem['location'][$this->languages[$j]] = "$DOCUMENT_ROOT/../" . str_replace('/_pl/', '/_' . $this->languages[$j] . '/', $buffer[$i]);
                }
                $elem['group'] = $groupname;
                $this->fileArray[] = $elem;
            }
        }
        else
        echo "<br>Brak pliku z list $lang_list_file!<br>";
    }

    /**
    * Pobranie zmiennych do tablicy CurrentVars
    *
    * Funkcja pobiera zmienne z pliku z definicjami jzykowymi i wpisuje je do tablicy
    * CurrentVars
    *
    * @param int $i Indeks tablicy FileArray, pod ktrym znajduje si cieka do pliku
    * zawierajcego zmienne pobierane do CurrentVars.
    */
    function readLangFileToArray($i, $lang_name){
        $lang_file = $this->fileArray[$i]['location'][$lang_name];
        if(is_file($lang_file)){
            global $lang;
            
//            $this->currentVars = array();
            $this->_includeLangFile($i, $lang_name);
            
            $tablica = array($this->lang);
            $tablica = $tablica[0];
            
            while(list($key, $val) = each($tablica)){
                if($this->fileArray[$i]['group'] != 'main')
                    $this->currentVars[$key]['group'] = $this->fileArray[$i]['group'];
                else{
                    for($j = 0; $j < count($this->groups); $j++){
                        if(ereg("^" . $this->groups[$j], $key)){
                            $this->currentVars[$key]['group'] = $this->groups[$j];
                        }
                    }
                }
                if(@$this->currentVars[$key]['group'] == '')
                    $this->currentVars[$key]['group'] = 'main';
                $this->currentVars[$key]['value'][$lang_name] = $val;
                $this->currentVars[$key]['location'][$lang_name] = $lang_file;
            }
//            print "<pre>";print_r($this->currentVars);print "</pre>";
        }
//        else
//        echo "<br>Brak pliku jzykowego $lang_file!<br>";
    }
    
    

    /**
    * Skopiowanie pliku do katalogu tymczasowego
    *
    * Plik zostaje skopiowany do katalogu tymczasowego w celu umoliwienia jego ponownego
    * "includowania", gdy zosta ju uprzednio zaczony poprzez "include_once" lub
    * "require_once"
    *
    * @param int $i Indeks tablicy FileArray, pod ktrym znajduje si cieka do kopiowanego
    * pliku. Zmienna ta zostaje rwnie dodana jako przedrostek do nazwy pliku w celu
    * uniknicia niepodanego nadpisania.
    */
    function _copyFileToTemp($i, $lang_name){
        $filename = basename($this->fileArray[$i]['location'][$lang_name]);
        if(!is_dir($this->temp_dir)){
            echo 'Brak katalogu ' . $this->temp_dir;
            return 0;
        }
        if(is_file($this->fileArray[$i]['location'][$lang_name])){
            copy($this->fileArray[$i]['location'][$lang_name], $this->temp_dir . "/$i" . '_' . $lang_name . '_' . $filename);
            return 1;
        }
    }
    
    /**
    * Usunicie pliku z katalogu tymczasowego
    *
    * Plik zostaje usunity z katalogu tymczasowego.
    *
    * @param int $i Indeks tablicy FileArray, pod ktrym znajduje si cieka do usuwanego
    * pliku.
    */
    function _deleteFileFromTemp($i, $lang_name){
        $filename = basename($this->fileArray[$i]['location'][$lang_name]);
        if(is_file($this->temp_dir . "/$i" . '_' . $lang_name . '_' . $filename))
        unlink($this->temp_dir . "/$i" . '_' . $lang_name . '_' . $filename);
    }
    
    /**
    * Zaczenie pliku
    *
    * Plik zostaje skopiowany do katalogu tymczasowego w celu umoliwienia jego ponownego
    * "includowania", gdy zosta ju uprzednio zaczony poprzez "include_once" lub
    * "require_once"
    *
    * @param int $i Indeks tablicy FileArray, pod ktrym znajduje si cieka do zaczanego
    * pliku.
    */
    function _includeLangFile($i, $lang_name){                
        if($this->_copyFileToTemp($i, $lang_name)){
            $filename = basename($this->fileArray[$i]['location'][$lang_name]);            
            if(is_file($this->temp_dir . "/$i" . '_' .$lang_name . '_' . $filename)){
                $file_include=$this->temp_dir . "/$i" . '_' . $lang_name . '_' . $filename;                                                                
                $lang =& new LangTmp;
                require($file_include);                    
                $this->lang=$lang;
            }
            $this->_deleteFileFromTemp($i, $lang_name);
        }
        else
            echo "Bd";
    }
    
    function prepareInsertQuery(){
        $query = '';
        $count = 0;
        for($i = 0; $i < count($this->fileArray); $i++){
            $this->currentVars = array();
            for($j = 0; $j < count($this->languages); $j++){
                $this->readLangFileToArray($i, $this->languages[$j]);
            }
            
            while(list($key, $val) = each($this->currentVars)){
                $uniloc = '';
                $q_lang = '';
                $q_lang_val = '';
                for($j = 0; $j < count($this->languages); $j++){
                    if($uniloc == '')
                        $uniloc = @$val['location'][$this->languages[$j]];
                    $q_lang .= "," . addslashes($this->languages[$j]) . " ";
                    $q_lang_val .= ",'" . addslashes(serialize(@$val['value'][$this->languages[$j]])) . "' ";
                }
                $uniloc = strstr(addslashes(substr($uniloc, 0, -16)), '/./htdocs/');
                $query .= "insert into lang (name, group_name, path $q_lang) values('" . addslashes($key) . "', '" . addslashes($val['group']) . "', '$uniloc' $q_lang_val);
                ";
                $count++;
            }
        }
     //   echo "[$count]";
        return $query;
    }
};
?>