<?php
/**
* Plik z definicj klasy Report
*
* Domylny skrypt obsugujcy generowanie raportw sprzeday.
* @author  lech@sote.pl
* @version $Id: report.inc.php,v 1.3 2004/07/08 11:17:25 lechu Exp $
* @package report
*/

/**
* Plik klasy SimpleBarChart - rysowanie wykresw supkowych
*/
require_once ("SimpleBarChart/include/simplebarchart.inc.php");

/**
* Gwna klasa obsugujca generowanie raportu sprzeday produktw
*
* Klasa generuje 4 rodzaje raportw:
* <ul>
*   <li>zestawienie najlepiej sprzedawanych produktw</li>
*   <li>zestawienie kategorii najlepiej sprzedawanych produktw</li>
*   <li>zestawienie producentw najlepiej sprzedawanych produktw</li>
*   <li>zestawienie liczby transakcji lub wartoci transakcji</li>
* </ul>
* Kade zestawienie moe zosta ograniczone zadanym przedziaem czasowym.
* 
* @package report
*/
class Report {
    
    /**
    * Pole okrelajce rodzaj raportu do wygenerowania.
    * Moe przyj jedn z czterech nastpujcych wartoci:
    * <ul>
    *   <li>"q_products" - zestawienie najlepiej sprzedawanych produktw</li>
    *   <li>"q_categories" - zestawienie kategorii najlepiej sprzedawanych produktw</li>
    *   <li>"q_producers" - zestawienie producentw najlepiej sprzedawanych produktw</li>
    *   <li>"q_transactions" - zestawienie liczny lub wartoci transakcji</li>
    * </ul>
    * Pole inicjowane konstruktorem
    * @var string
    */
    var $report_type;
    
    /**
    * Tablica wartoci przekazanych do kostruktora klasy SimpleBarChart.
    * Posiada dwa pola:
    * <ul>
    * <li>title - tytu wykresu w postaci HTML</li>
    * <li>data - tablica danych do supkw wykresu. Kluczami s nazwy supkw,
    * wartociami - wartoci supkw</li>
    * </ul>
    * @var array
    */
    var $data = array();
    
    /**
    * Tablica danych pobranych z formularza
    * 
    * Tablica inicjowana jest jest w konstruktorze poprzez dokadn kopi
    * tablicy $_REQUEST. Dla uytku klasy wykorzystywane s nastpujce pola:
    * <ul>
    * <li>date_data - tablica, ktrej kluczami s kolejne indeksy typu int (od zera),
    * a wartociami - tablice reprezentujce daty o nastpujcej strukturze:
    *   <ul>
    *       <li>y - rok w postaci 4-cyfrowej</li>
    *       <li>m - dzie w postaci 2-cyfrowej</li>
    *       <li>d - miesic w postaci 2-cyfrowej</li>
    *   </ul>
    * Poszczeglne elementy tablicy oznaczaj rne terminy, np. dolne ograniczenie
    * czasowe zestawienia, grne ograniczenie czasowe zestawienia, itp. Ich kolejno
    * i interpretacja zaley od rodzaju raportu.
    * </li>
    * <li>producer_data - tablica, ktra w polu "id" zawiera 0 lub identyfikator producenta
    * o formacie "IdProducenta|NazwaProducenta" - dotyczy tylko raportu produktw
    * </li>
    * <li>category_data - tablica, ktra w polu "id" zawiera 0 lub identyfikatory
    * kategorii 1 i kategorii 2 (kategorii gwnej produktu i podkategorii)
    * o formacie "IdKategorii1_IdKategorii2|NazwaKategorii1_NazwaKategorii2"
    *  - dotyczy tylko raportu produktw
    * </li>
    * <li>amount_data - tablica, ktra w polu "amount" wskazuje, ile pierwszych
    * pozycji ma zosta ujtych w zestawieniu
    *  - nie dotyczy tylko raportu transakcji</li>
    * <li>transaction_form - tablica, ktra w polu "division" okrela
    * rodzaj zestawienia raportu transakcji:
    *   <ul>
    *     <li>d_1 - podzia na miesice</li>
    *     <li>d_2 - podzia na dni</li>
    *     <li>d_3 - podzia na godziny</li>
    *   </ul>
    *   natomiast w polu "values" okrela, jakie wartoci bd prezentowane na wykresie
    *   transakcji:
    *   <ul>
    *     <li>1 - liczby transakcji</li>
    *     <li>2 - sumy wartoci transakcji</li>
    *   </ul>
    *    - dotyczy tylko raportu transakcji</li>
    * </ul>
    * @var array
    */
    var $form_data;

    /**
    * Konstruktor klasy
    *
    * Inicjuje pola report_type i form_data
    * 
    * @param string $report_type Warto inicjujca pole report_type
    * @return none
    */
    function Report($report_type = '') {
        $this->report_type = $report_type;
        if($this->report_type == '')
            $this->report_type = 'q_products';
        
        $this->form_data = @$_REQUEST;
        return ;
    }
    
    /**
    * Metoda poprawiajca bdnie wpisan dat
    * 
    * Metoda zamienia dat np. 31 czerwca 2004 na 1 lipca 2004
    *
    * @param array $ar_date Data do poprawienia. Tablica ma 3 pola:
    * <ul>
    *   <li>y - rok w formacie 4-cyfrowym</li>
    *   <li>m - rok w formacie 2-cyfrowym</li>
    *   <li>d - rok w formacie 2-cyfrowym</li>
    * </ul>
    *
    * @return array Tablica z poprawion dat w tym samym formacie, co parametr wejciowy
    */
    function _correctDate($ar_date) {
        if(count($ar_date) < 3)
            return $ar_date;
        $time = mktime(0, 0, 0, (int)$ar_date['m'], (int)$ar_date['d'], (int)$ar_date['y']);
        $ar_result['m'] = date('m', $time);
        $ar_result['d'] = date('d', $time);
        $ar_result['y'] = date('Y', $time);
        return $ar_result;
    }
    
    /**
    * Pobranie danych z bazy
    *
    * Metoda pobiera dane z bazy i przeksztaca je do formatu przystosowanego
    * dla klasy wywietlajcej wykres SimpleBarChart (dane zostaj zapisane do
    * pola "data" - patrz opis pola)
    * @return none
    */
    function _prepareData() {
        $data = array();
        global $db;
        include_once("./include/" . $this->report_type . ".php");
        $this->data = $data;
        return ;
    }
    
    /**
    * Generowanie formularza do raportu
    * 
    * Funkcja tworzy formularz, do ktrego mona wprowadzi wartoci precyzujce
    * dane, ktre chcemy pokaza w raporcie, np. przedzia czasowy
    *
    * @return none
    */
    function _drawForm() {
        global $db;

        include('./html/' . $this->report_type . '_form.html.php');
        return ;
    }
    
    /**
    * Generowanie caego raportu
    * 
    * Funkcja realizuje pene wygenerowanie raportu: formularza, pobranie danych
    * do wykresu i narysowanie wykresu
    *
    * @return none
    */
    function draw() {
        $this->_drawForm();
        $this->_prepareData();
        echo '<br>';
        $chart =& new SimpleBarChart($this->data, 400);
        $chart->bar_unit_image = 'unit_violet_3d.jpg';
        $chart->border = 1;
        $chart->draw();
        return ;
    }
}

?>