<?php
/**
* Obsuga plikw aktualizacji.
* Odczytanie plikw do aktualizacji, weryfikacja, aktualizacja.
*
* @author  m@sote.pl
* @version $Id: upgrade.inc.php,v 1.16 2004/10/18 15:01:22 maroslaw Exp $
* @package upgrade
*/

/**
* Dodaj obsug FTP
*/
require_once ("include/ftp.inc.php");

/**
* Dodaj obsug plikw CVS
*/
require_once ("include/cvs.inc.php");

/**
* Dodaj obsug generowania sum kontrolnych plikw
*/
require_once ("include/file_checksum.inc.php");

/**
* Maksymalna ilo zagniedze katalogw przy zakadaniu nowych cieek.
* Ilo katalogw przegldanych "w gr" w celu znalezienia odpowiedniej cieki, w ktrej
* mona zakada wsakzane katalogi.
*/
define ("MKDIR_MAX_COUNT",8);

/**
* Okrelenie do jakiej bazy sum kontrolnych jest zapisywana suma kontrolna. UPGRADE_FILE_MOD -> mod.md5
*/
define ("UPGRADE_FILE_SUM",1);

/**
* Okrelenie do jakiej bazy sum kontrolnych jest zapisywana suma kontrolna. UPGRADE_FILE_SUM -> sum.md5
*/
define ("UPGRADE_FILE_MOD",2);

/**
* Obsuga plikw "Diff"
* @package upgrade
*/
class Diff {
    
    /**
    * @var array wiersze z aktualizacji
    * @access private
    */
    var $_file_lines=array();
    
    /**
    * @var array lista plikw do aktualizacji
    */
    var $_files=array();
    
    /**
    * @var int licznik zaoonych katalogw, przy tworznieu odpowiedniej cieki do pliku (zobacz MKDIR_MAX_COUNT)
    */
    var $_mkdirCount=0;
    
    /**
    * Konstruktor
    *
    * @param string $file plik z aktualizacj
    * @return bool true plik poprawnie zaczony, false w p.w.
    */
    function Diff($file) {
        if (file_exists($file)) {
            $fd=fopen($file,"r");
            $data=fread($fd,filesize($file));
            fclose($fd);
            $this->_file_lines=split("\n",$data);
            return true;
        }
        return false;
    } // end Diff()
    
    /**
    * Odczytaj list plikw do aktualizacji
    * @return array
    */
    function _getFiles() {
        $files=array();
        reset($this->_file_lines);
        foreach ($this->_file_lines as $_line) {
            if (ereg("^diff -ruN",$_line)) {
                $tab=split(" ",$_line,4);
                if (! empty($tab[3])) $files[]=$tab[3];
            }
        } // end foreach
        $this->_files=$files;
        return $files;
    } // end _getFiles()
    
    /**
    * Odczytaj zawartoc wskazanego pliku z pakietu aktualizcyjnego.
    *
    * @param string nazwa pliku + cieka w wzgldem $DOCUMENT_ROOT/../
    * @return string zawartoc pliku
    */
    function getFile($file) {
        if (empty($this->_file_lines)) return;
        reset($this->_file_lines);$file_source='';$_read=false;$_start=false;
        foreach ($this->_file_lines as $_line) {
            if (ereg(" ".$file."$",$_line)) $_read=true;
            if ((! (ereg(" ".$file."$",$_line))) && ((ereg("^diff "."-ruN ",$_line)))) { $_read=false; $_start=false; }
            if (ereg("\ No newline "."at end of file",$_line)) { $_read=false; $_start=false; }
            if (($_read) && ($_start)) {
                $line=preg_replace("/^\+/","",$_line); // usu znak +, ktry dodaje diff na pocztku kadej linii
                $file_source.=$line."\n";
            }
            if (($_read) && (! $_start)) {
                if (ereg("^@@ ",$_line)) $_start=true;
            }
        } // end foreach()
        return $file_source;
    } // getFile()
    
} // end class Diff

/**
* Klasa aktualizacja sklepu na bazie plikw "diff".
* @package upgrade
*/
class DiffUpgrade extends Diff {
    
    /**
    * @var int numer wersji upgrade'u, dla kadego upgradu numer powinien by nowy numer CVS
    */
    var $_version=0;
    
    /**
    * @var string zmienna zawierajca plik zawierajcy pakiet upgrade'u (nazwa pliku z uaktualneiniem)
    * @access public
    */
    var $upgradeFile='';
    
    /**
    * Lista podkatalogw z gwnego katalogu sklepu, dla ktrych zmieniana jest lokalizacja docelowa
    * Dot. instalacji na home.pl
    */
    var $_base_dirs=array("config","include","lib","bin");
    
    /**
    * Kondtruktor
    * @see Diff::Diff()
    */
    function DiffUpgrade($file) {
        $this->upgradeFile=$file;
        preg_match("/[0-9]([0-9]+)\./",basename($file),$matches);
        if (! empty($matches[1])) $this->_version=intval($matches[1]);
        else $this->version=date("YmdHi");
        return $this->Diff($file);
    } // end DiffUpgrade()
    
    /**
    * Wywietl liste plikw do aktualizacji
    * @return none
    */
    function showListFiles() {
        if  (empty($this->_files)) $this->_getFiles();
        reset($this->_files);
        print "<ul>\n";
        foreach ($this->_files as $_file) {
            $file_url=urlencode($_file);
            print "<li>$_file</li>\n";
        } // end foreach
        print "</ul>\n";
        return;
    } // end showListFiles()
    
    /**
    * Wykonaj upgrade
    *
    * @return bool true upgrade wykonany poprawnie, false w p.w.
    */
    function doUpgrade() {
        global $ftp,$DOCUMENT_ROOT;
        global $lang;
        
        $this->_getFiles();
        if  (empty($this->_files)) $this->_getFiles();
        $ftp->connect();
        print "<table cellspacing=1 cellpadding=0 border=0>\n";
        foreach ($this->_files as $_file) {
            print "<tr>\n";
            // spawdz, czy plik mozna zaktualizowac
            $result_upgrade_test=$this->_upgradeTestFile($_file);
            if ($result_upgrade_test==0) {
                if ($this->upgradeFile($_file)) {
                    print "<td>$_file</td>\n";
                }
            } else {
                print "<td>$_file ";
                switch ($result_upgrade_test) {
                    case "1": $color="red";
                    break;
                    case "2": $color="green";
                    break;
                    case "3": $color="orange";
                    break;
                    case "4": $color="brown";
                    break;
                    default: $color="black";
                    break;
                }
                print "</td>\n";
                print "<td>\n";
                print " <font color=\"$color\">".$lang->upgrade_file_status[$result_upgrade_test]."</font>";
                print "</td>\n";
                if ($result_upgrade_test!=2) {
                    print "<td>\n";
                    // poka formularze do wykrycia modyfikacji pliku i "wyleczenia/pogodzenia" zmian
                    print "      <form action=repair.php method=POST target=window>\n";
                    print "      <input type=hidden name=file[name] value='".$_file."'>\n";
                    print "      <input type=hidden name=file[upgrade_file] value='".$this->upgradeFile."'>\n";
                    print "      <input type=hidden name=file[path] value='".$this->_files['path']."'>\n";
                    print "      <input type=hidden name=file[upgrade] value='".$this->_files['upgrade']."'>\n";
                    print "      <input type=submit value='$lang->upgrade_file_repair' onclick='window.open(\"\", \"window\", \"width=800, height=600, status=no, toolbar=no, scrollbars=yes, resize=1\")'>\n";
                    print "      </form>\n";
                    print "</td>\n";
                }
                print "</tr>\n";
            }
        } // end foreach()
        print "</table>\n";
        
        global $__database_md5;
        
        // zapisz now baz sum kontrolnych
        if (FileChecksum::saveDat()) {
            print "<p>$lang->upgrade_checksums_upgraded <p>";
        } else {
            print "<p><font color=\"red\">$lang->upgrade_checksums_upgrade_error</font><p>\n";
        }
        
        $ftp->close();
        
        return true;
    } // end doUpgrade()
    
    /**
    * Sprawd czy mona zaktualizowa plik
    * Weryfikacja czy plik nie zosta zmieniony przez klienta i czy wersja pliku w sklepie nie jest
    * nowsza od wersji instalowanej. Dodatkowo system sprawdza, czy plik na dysku nie jest zgodny z plikiem
    * w upgrade. Jeli tak to mimo rznych sum kontrolnych zwraca wynik pozytywny.
    *
    * @param string $file plik
    *
    * @access private
    * @return int     0 - mona aktualizowa plik,
    *                 1 - plik zmieniony przez usera,
    *                 2 - nowsza wersja pliku
    *                 3 - wykryta indywidualna modyfikacja,
    *                 4 - plik zaktualizowany w poprzedniej aktualizacji (wykryto wczesniej indywidualna modyfikacje /3/)
    */
    function _upgradeTestFile($file) {
        global $DOCUMENT_ROOT;
        global $shop;
        
        if ($shop->home!=1) {
            $_file_root_path="$DOCUMENT_ROOT/../../$file";
        } else {
            $_file_root_path=preg_replace("/^soteesklep3/","",$file);
        }
        
        if (! file_exists($_file_root_path)) {            
            return (0);
        }
        $_file_base_path=ereg_replace("soteesklep3",".",$file);  // cieka typu ./admin/go/_edit.index.php
        
        // plik z systemu plikow
        $cvs =& new CVS($_file_root_path,CVS_FILE);
        $file_version_cvs=$cvs->getVersion();                    // numer wersji CVS np. 2.9
        $file_version=$cvs->idCVS2Int($file_version_cvs);        // numer wersji CVS INT np. 29
        
        // odczytaj sume kontrolna z systemu plikow
        $fc =& new FileChecksum($_file_root_path,FILE_CHECKSUM_FILE);
        $file_checksum=$fc->getChecksum();
        if ($fd=@fopen($_file_root_path,"r")) {
            $_file_source=fread($fd,filesize($_file_root_path));
            $file_source_from_path=trim($_file_source);
        } else $file_source_from_path='';
        
        // plik z upgrade
        $file_source_from_upgrade=trim($this->getFile($file));
        $cvs =& new CVS($file_source_from_upgrade,CVS_SOURCE);
        $file_version_upgr_cvs = $cvs->getVersion();
        $file_version_upgr = $cvs->idCVS2Int($file_version_upgr_cvs);
                        
        $file_checksum_dat=$fc->getChecksumFromDAT($_file_base_path);
        
        // sprawd czy plik w pakiecie upgrade jest taki sam jak plik na dysku, jeli tak to zezwalaj na aktualizacj
        if ($file_source_from_path==$file_source_from_upgrade) {
            return (0);
        }
        
        // zapamitaj  w przestrzeni globalnej obiektu wartoci plikw, z ktrymi wystpi problem
        $this->_files=array();
        $this->_files['path']=urlencode($file_source_from_path);
        $this->_files['upgrade']=urlencode($file_source_from_upgrade);
        
        if (ereg("\\\@"."v",$file_source_from_path)) {
            if ($this->_testModFile($_file_base_path,$file_checksum)) return (4);
            else return (3);  // indywidualna  modyfikacja skryptu wykonana przez SOTE
        }
        if (empty($file_checksum_dat)) {            
            return (0);               // brak sumy w pliku bazy sum
        }
        if ($file_checksum_dat!=$file_checksum) {
            // @todo sprawdzic czy klient nie zaktualizowal wczesniej zmodyfikoany plik
            // tj. czy jest odpowiedni wpis w mod.md5 i czy suma kontrolna tam zawarta
            // jest zgodna z sum kontroln pliku na dysku
            // (jest tak return (4) else return (1))
            if ($this->_testModFile($_file_base_path,$file_checksum)) return (4);
            else return (1);  // wykryto modyfikacj pliku
        }
        if ($file_version>$file_version_upgr) return (2);        // ma dysku jest nowsza wersja pliku
                
        return (0);
    } // end _upgradeTestFile()
    
    /**
    * Sprawd czy wskazany plik by wczeniej zmodyfikowany i zmiany zostay zatwierdzone przez admina/developera
    *
    * @param string $file          cieka do pliku np. ./admin/go/_order/html/edit.html.php
    * @param string $file_checksum suma kontrolna aktualnego pliku na dysku
    * @access private
    * @return bool true - plik zosta wczeniej zmodyfikowany i zatwierdzony, false w p.w.
    */
    function _testModFile($file,$file_checksum) {
        global $DOCUMENT_ROOT;
        
        $file_md5="$DOCUMENT_ROOT/../mod.md5";
        $fd=fopen($file_md5,"r");
        $data=fread($fd,filesize($file_md5));
        fclose($fd);
        
        $lines=split("\n",$data);
        foreach ($lines as $line) {
            $tab=split("  ",$line,2);
            if (! empty($tab[1])) {
                if ((trim($tab[1])==$file) && (trim($tab[0])==$file_checksum)) return true;
            }
        }
        
        return false;
    } // end _testModFile()
    
    /**
    * Aktualizuj wskazany plik
    *
    * @param string $file        plik wzgldem $DOCUMENT_ROOT/../ np. soteesklep3/htdocs/index.php
    * @param string $file_source zawarto pliku, jesli wartoc nie jest domylnie pobierana z pakietu upgrade
    * @param int    $type         UPGRADE_FILE_SUM - zapisz dane w sum.md5, UPGRADE_FILE_MOD - nie zpisuj w sum.md5
    *
    * @access public
    * @return bool true upgrade OK, false w p.w.
    */
    function upgradeFile($file,$file_source='',$type=UPGRADE_FILE_SUM) {
        global $ftp,$DOCUMENT_ROOT,$config;
        global $shop;
        
        if (empty($file_source)) {
            $file_source=$this->getFile($file);             // pobierz zawarto wskazanego pliku z pakietu upgrade
        }
        
        $file_name=basename($file);                         // odczytaj nazw pliku
        $file_name_copy=$this->_version."-".$file_name;     // nazwa kopii pliku
        $file_dir=dirname($file);                           // katalog wzgldem $DOCUMENT_ROOT/../ np. soteesklep3/htdocs
        
        if (@$shop->home!=1) {
            $file_current=$DOCUMENT_ROOT."/../../$file_dir/$file_name";
        } else {
            // home.pl
            // rozrnij katalogi soteesklep3/admin/include i soteesklep3/include
            // na home.pl /include wskazuje na soteesklep3/admin.include a nie na gwny katalog
            // odwoania do gwnego katalogu, przekieruj na soteesklep3/admin/htdocs/base/include
            foreach ($this->_base_dirs as $dir) {
                if (ereg("^soteesklep3/".$dir,$file_dir)) {
                    $file_dir=preg_replace("/^soteesklep3\/".$dir."/",'soteesklep3/admin/htdocs/base/'.$dir,$file_dir);
                }
            }
            $file_current=preg_replace("/^\/soteesklep3/","","/$file_dir/$file_name");
            $file_current=preg_replace("/^\/admin/","",$file_current);
        }
        
        $file_new="$DOCUMENT_ROOT/tmp/upgrade_$file_name";
        if (@$shop->home!=1) {
            $file_copy=$DOCUMENT_ROOT."/../../$file_dir/$file_name_copy";
        } else {
            // home.pl
            $file_copy=preg_replace("/^\/soteesklep3/","","/$file_dir/$file_name_copy");
            $file_copy=preg_replace("/^\/admin/","",$file_copy);
        }
        $this->_file_copy=$file_copy;
        
        // zapisz zrodla do tymczasowego pliku
        if ($fd=fopen($file_new,"w+")) {
            fwrite($fd,$file_source,strlen($file_source));
            fclose($fd);
            if ((file_exists($file_current)) && (! file_exists($file_copy))) {
                // zrob kopie starej wersji pliku
                $ftp->put($file_current,$config->ftp['ftp_dir']."/../$file_dir",$file_name_copy);
            }
            // aktualizuj katalogi, za odpowiednie katalogi, jeli nie ma ich w obecnym drzewie programu
            $this->_mkDir($file_dir);
            // aktualizuj plik
            $ftp->put($file_new,$config->ftp['ftp_dir']."/../$file_dir",$file_name);
            unlink($file_new);
            
            if ($type==UPGRADE_FILE_SUM) {
                // oblicz sum kontroln zainstalowanego pliku  i zapamitaj j w bazie sum kontrolnych
                $file_dir2=preg_replace("/^soteesklep3/","",$file_dir);
                $file_fc=".".$file_dir2."/$file_name";                  // np. ./admin/index.php
                $file_fc_path=$DOCUMENT_ROOT."/..".$file_dir2."/$file_name";  // np. /home/sklep/soteesklep3/admin/index.php
                $fc =& new FileChecksum($file_fc_path,FILE_CHECKSUM_FILE);
                $file_checksum=$fc->getChecksum();
                $fc->set($file_fc,$file_checksum);
            }
        } else return false;
        
        return true;
    } // end upgradeFile()
    
    /**
    * Odczytaj pen sciek (z katalogiem sklepu) do kopii pliku (np. soteesklep3/htdocs/8-index.php)
    *
    * \@global string $this->_file_copy np. /usr/home/maroslaw/MYCVS2/soteesklep2/admin/../../soteesklep3/htdocs/8-index.php
    * @return string np. soteesklep3/htdocs/8-index.php
    */
    function getLastFileCopyName() {
        if (! empty($this->_file_copy)) {
            preg_match("/soteesklep[0-9]{1}(.*)$/",$this->_file_copy,$matches);
            if (! empty($matches[1])) {
                return ereg_replace('/admin/../../','',$matches[1]);
            }
        }
        return '';
    } // end getlastFileCopyName()
    
    /**
    * Za wskazany katalog, jesli nie ma go w systemie plikw
    *
    * @param string $dir cieka do katalogu wzgldem soteesklep3
    * @return bool
    */
    function _mkDir($dir) {
        global $DOCUMENT_ROOT,$ftp,$config;
        
        
        $dir=preg_replace("/^soteesklep[0-9]\//","",$dir);
        
        if (! is_dir("$DOCUMENT_ROOT/../".$dir)) {
            $this->_mkRecSubDir($dir);
            $ftp->mkdir($config->ftp['ftp_dir']."/".$dir);
        }
        return true;
    } // end _mkDir()
    
    /**
    * Za rekurencyjnie wskazany katalog (poza ostatnim)
    * Funkcja rekurencyjna.
    *
    * @param string $dir katalog wzgldem gwnego katalogu soteesklep3
    *
    * @access private
    * @return bool true - katalog zaozony, false - w p.w.
    */
    function _mkRecSubDir($dir) {
        global $ftp,$config,$DOCUMENT_ROOT;
        
        // obetnij ostatni katalog i sprawd, czy poziom wyej moa zaoy podkatalog
        $last_dir=basename($dir);
        
        $dir2=$dir;
        
        $dir=substr($dir,0,strlen($dir)-strlen($last_dir)-1);
        $test_dir="$DOCUMENT_ROOT/../$dir";
        
        if (is_dir($test_dir)) {
            $ftp->mkdir($config->ftp['ftp_dir']."/".$dir2);
            return true;
        } else {
            if ($this->_mkdirCount<MKDIR_MAX_COUNT) {
                $this->_mkdirCount++;
                $this->_mkRecSubDir($dir);
                return;
            }
        }
    } // end _mkRecSubDir()
    
    /**
    * Odczytaj numer upgrade'u
    * @return int numer wersji aktualizacji
    */
    function getVersion() {
        return $this->_version;
    } // end getVersion();
    
    
} // end class DiffUpgrade

?>
